
;+
; NAME:
;
; filename2time
;
; PURPOSE:
;
; To convert a SOHO/SUMER filename into a time.
;
; CATEGORY:
;
; Data handling.
;
; CALLING SEQUENCE:
;
; time = filename2time(filename)
; 
; INPUTS:
;
; Any SUMER filename.
;
; OPTIONAL INPUTS:
;
; None.
;       
; KEYWORD PARAMETERS:
; 
; None.
;
; OUTPUTS:
;
; A seven element array containing the time. The elements of the array
; are the hour, minute, second, millisecond, day, month, year that an
; observation began. Note that milliseconds is always zero since it is
; not specified in the filename.
;
; OPTIONAL OUTPUTS:
;
; None.
;
; COMMON BLOCKS:
;
; None.
;
; SIDE EFFECTS:
;
; None.
;
; RESTRICTIONS:
;
; The input filename must contain the extension. This routine really
; should be subsumed into gt_time.
;
; PROCEDURE:
;
; The routine parses the 13 characters before the file extension
; (".fits") to determine the time.
;
; EXAMPLE:
;
; IDL> filename = '$SUMER_DATA/sum_960513_085055.fits'
; IDL> time = filename2time(filename)
; IDL> print,time
;      8      50      55       0      13       5      96
;
; MODIFICATION HISTORY:
;
;   HPW 24-JUL-1996:
;
;   HPW 04-OCT-1996: Added the genx file extension.
;
;-

function filename2time,input_filename

;; return to main level on error
on_error,1

filename = strlowcase(input_filename)
ss    = str_sep(filename,".")
ext   = ss(n_elements(ss)-1)
pos   = strpos(filename,"."+ext)
match = where(pos ne -1,count)

;; milliseconds are not specified in the filename
ms = 0 				

if (count gt 0) then begin

  times = intarr(7,count)
  for i=0,count-1 do begin
    ;; get date and time from filename
    date = strmid(filename(match(i)),pos(i)-13,13)
    
    ;; parse string
    yy = strmid(date,0,2) & mm = strmid(date,2,2) & dd = strmid(date,4,2)
    h  = strmid(date,7,2) & m  = strmid(date,9,2) & s  = strmid(date,11,2)

    ;; [hour,minute,second,millisecond,day,month,year]
    times(*,i) = fix([h,m,s,ms,dd,mm,yy])
  endfor

endif else begin
  message,'Unable to parse filename(s).'
endelse

return,reform(times)
end
