
;+
; NAME:
;
; mk_query_genx
;
; PURPOSE:
;
; To retrieve information about data stored in a SUMER/SOHO data file
; that has been written with wrt_sumer. See mk_query for more
; details. This routine is called by the routine mk_query, which also
; works on the original FITS files. The data structure returned by
; mk_query_genx is more extensive, however.
;
; Note that this routine always displays the first order wavelengths
; unless the keyword default_wave is specified.
;
; CATEGORY:
;
; Data retrieval.
;
; MODIFICATION HISTORY:
;
;   HPW 18-JUL-1997: Orginal version.
;
;   HPW 04-FEB-1998: Modified to return the output as a string. Useful
;                    for printing the contents of files.
;
;-

pro mk_query_genx,filename,query,silent=silent,default_wave=default_wave,$
                      header=header

;; ----------------------------------------------------------------------
;; Check user inputs
;; ----------------------------------------------------------------------

for n=0,n_elements(filename)-1 do begin
  if not(file_exist(filename(n))) then $
      message,"Specified file does not exist : "+filename(n)

  if sgt_file_type(filename(n),/SUMER) ne "SUMERGENX" then $
      message,"Specified file is not a SUMER genx file : "+filename(n)
endfor

;; ----------------------------------------------------------------------
;; Define a structure to hold the query results
;; ----------------------------------------------------------------------

q_struct = {name:		  '',$	 ;; name of the column
            extension:		   0,$   ;; binary table extension number
            column:		   0,$   ;; column number
            dims:   	   intarr(3),$   ;; dimensions of the column
            dims_str: 	          '',$   ;; dimensions of the column
            solar_x_min:         0.0,$   ;; min x pointing of the column
            solar_x_max:         0.0,$   ;; max x pointing of the column
            solar_y_min:         0.0,$   ;; min y pointing of the column
            solar_y_max:         0.0,$   ;; max y pointing of the column
            wave_min:		 0.0,$   ;; min wavelength of the column
            wave_max:		 0.0,$   ;; max wavelength of the column
            ;; EXTENSIONS TO ORIGINAL QUERY STRUCTURE
            wave_min1:		 0.0,$   ;; min first-order wavelength
            wave_max1:		 0.0,$   ;; max first-order wavelength
            wave_min2:		 0.0,$   ;; min second-order wavelength
            wave_max2:		 0.0,$   ;; max second-order wavelength
            ref_pix:               0,$   ;; reference pixel
            ref_val:             0.0,$   ;; reference pixel
            delta:               0.0,$   ;; reference pixel
            detector:             '',$   ;; detector
            max_exptime:         0.0,$   ;; max exposure time
            min_exptime:         0.0,$   ;; min exposure time
            slit:                 '',$   ;; slit
            slit_n:                0,$   ;; slit
            filename:             '',$   ;; location of the file
            file_name:            ''}

;; ----------------------------------------------------------------------
;; Accumulate query data
;; ----------------------------------------------------------------------

init = 1

for i=0,n_elements(filename)-1 do begin

  rd_sumer,filename(i),index,/nodata
  names  = tag_names(index)
  n_cols = n_elements(names) - 1
  q      = replicate(q_struct,n_cols)

  for j=0,n_cols-1 do begin
    ;; get data from index structure
    exptime  = sgt_exptime(index.(j))
    dim      = sgt_dims(index.(j))
    delta    = sgt_delta(index.(j))
    ref_val  = sgt_refval(index.(j))
    ref_pix  = sgt_refpix(index.(j))
    solar_x  = sgt_solar_x(index.(j))
    solar_y  = sgt_solar_y(index.(j))
    wave     = sgt_wave(index.(j))

    ;; query structure
    q(j).name        = names(j)
    q(j).column      = j + 1
    q(j).dims        = dim
    q(j).dims_str    = "("+trim(dim(0))+","+trim(dim(1))+","+trim(dim(2))+")"
    q(j).solar_x_min = min(solar_x)
    q(j).solar_x_max = max(solar_x)
    q(j).solar_y_max = min(solar_y)
    q(j).solar_y_min = max(solar_y)
    q(j).wave_max    = max(wave)
    q(j).wave_min    = min(wave)

    q(j).filename = filename(i)
    q(j).detector = sgt_detector(index,/text)
    q(j).slit     = sgt_slit(index,/text)
    q(j).slit_n   = sgt_slit(index)
    q(j).ref_val  = ref_val(0)
    q(j).ref_pix  = ref_pix(0)
    q(j).delta    = delta(0)
    q(j).max_exptime = max(exptime)
    q(j).min_exptime = min(exptime)

    ;; replace slashes with %
    file_name = filename(i)
    file_name = str_replace(file_name,'/','%')
    file_name = str_replace(file_name,'\','%')
    file_name = str_sep(file_name,'%')
    file_name = file_name(n_elements(file_name)-1)
    q(j).file_name = file_name    
    print,file_name


    ;; determine order of wavelengths from delta
    if q(j).delta lt 0.04 then wave2firstorder  = 2 else wave2firstorder  = 1
    if q(j).delta lt 0.04 then wave2secondorder = 1 else wave2secondorder = 0.5
    q(j).wave_max1 = max(wave)*wave2firstorder
    q(j).wave_min1 = min(wave)*wave2firstorder
    q(j).wave_max2 = max(wave)*wave2secondorder
    q(j).wave_min2 = min(wave)*wave2secondorder
  endfor

  ;; display information
  fn = filename(i)
  sl = strlen(filename(i))
  if sl gt 50 then fn = "... "+strmid(filename(i),sl-50,50)
  s = strarr(n_cols+7)

  s(0) = 'File       : '+fn
  s(1) = 'Detector   : '+q(0).detector
  s(2) = 'Slit       : '+q(0).slit
  s(3) = 'MAX Exptime: '+trim(max(q.max_exptime),'(f10.1)')+" sec"
  s(4) = 'MIN Exptime: '+trim(min(q.min_exptime),'(f10.1)')+" sec"
  s(5) = string(format='(a2,a10,a14,2a8,4a9)',$
               'C','Name','Dimensions','Wmin',$
               'Wmax','Xmin','Xmax','Ymin','Ymax')
  for j=0,n_cols-1 do begin
    name = q(j).name
    if keyword_set(default_wave) then begin
      wmin = q(j).wave_min & wmax = q(j).wave_max
    endif else begin
      wmin = q(j).wave_min1 & wmax = q(j).wave_max1
    endelse
    if (strlen(name) ge 11) then name = strmid(name,0,6)+'...'
    s(j+6) = string(format='(i2,a10,a14,2f8.2,4f9.2)',j+1,name,$
                    q(j).dims_str,wmin,wmax,$
                    q(j).solar_x_min,q(j).solar_x_max,$
                    q(j).solar_y_min,q(j).solar_y_max)
  endfor    
  s(n_cols+6) = ' '
  if not(keyword_set(silent)) then for j=0,n_elements(s)-1 do print,s(j)
  
  ;; concatenate query structure
  if init then begin
    query  = q 
    header = s
    init   = 0
  endif else begin
    query = concat_struct(query,q)
    header = [s,header]
  endelse

endfor


return
end
    
