
;+
; NAME:
;
; rd_sumer_genx
;
; PURPOSE:
;
; To read data files written with wrt_sumer. Called by rd_sumer. See
; rd_sumer for detailed documentation.
;
; CATEGORY:
;
; Data retrieval.
;
; MODIFICATION HISTORY:
;
;   HPW 17-JUL-1997: Orginal version.
;
;-

pro rd_sumer_genx,filename,index,data,nodata=nodata,data_cols=data_cols

;; ----------------------------------------------------------------------
;; Check user inputs
;; ----------------------------------------------------------------------

;; check file type 
file_type = sgt_file_type(filename)
if (file_type ne 'GENX') then $
    message,'Input file name must be either a GENX file'

;; check for existence of the file
if not(file_exist(filename)) then $
    message," specified file doesn't exist: "+filename

;; open filename
openr,unit,filename,/xdr,/get_lun

;; read version number
version = 0L
readu,unit,version

;; check version number
if not(version eq 999 or version eq 998) then $
    message,"This routine only reads SUMER files written with wrt_sumer."

;; ----------------------------------------------------------------------
;; Read file
;; ----------------------------------------------------------------------

case version of 

  ;; ------------------------------------------------------------
  ;; VERSION #999 
  ;; can't read individual data columns - data_cols has no effect
  ;; ------------------------------------------------------------

  999: begin
    
    ;; read index and data structures directly
    index = build_str(unit)
    readu,unit,index
    
    ;; read data
    if not(keyword_set(nodata)) then begin
      data = build_str(unit)
      readu,unit,data
    endif
    
    free_lun,unit
    return
  end

  ;; ------------------------------------------------------------
  ;; VERSION #998
  ;; ------------------------------------------------------------

  998: begin

    ;; ------------------------------------------------------------
    ;; INDEX
    ;; ------------------------------------------------------------

    ;; read index
    file_index = build_str(unit)
    readu,unit,file_index
    
    ;; index for requested data
    names  = tag_names(file_index)
    n_cols = n_elements(names) - 1 
    
    if keyword_set(data_cols) then begin
      ;; return selected data columns
      init      = 1
      read_data = make_array(n_cols,/int,value=0)
      
      for n=0,n_elements(data_cols)-1 do begin
        nn = data_cols(n) - 1
        if nn ge 0 and nn lt n_cols then begin
          read_data(nn) = 1
          if init then begin
            index = create_struct(names(nn),file_index.(nn))
            init  = 0
          endif else index = create_struct(index,names(nn),file_index.(nn))
        endif else message,"data_cols improperly specified."
      endfor
      index = create_struct(index,"gen",file_index.gen)
      
    endif else begin
      ;; return all data columns
      read_data = make_array(n_cols,/int,value=1)
      index     = file_index
    endelse

    ;; ------------------------------------------------------------
    ;; DATA
    ;; ------------------------------------------------------------

    if not(keyword_set(NODATA)) then begin
      
      n_cols = 0L
      readu,unit,n_cols
      
      str_def   = "{dummy"
      col_size  = lonarr(n_cols)
      
      for n=0,n_cols-1 do begin
        ;; read desciption of data columns
        n_x = 0L & n_y = 0L & n_exp = 0L & n_char = 0L
        readu,unit,n_x
        readu,unit,n_y
        readu,unit,n_exp
        readu,unit,n_char
        
        col_size(n) = n_x*n_y*n_exp*4 ;; always floats
        tag_name = string(replicate(32B,n_char))
        readu,unit,tag_name
        
        if (read_data(n)) then begin
          str_def = str_def + ","+tag_name+": "+$
              "fltarr("+trim(n_x)+","+trim(n_y)+","+trim(n_exp)+")"
        endif
      endfor
      str_def = str_def +"}"
      
      ;; make data structure
      data    = make_str(str_def)
      
      ;; read data
      i = 0
      for n=0,n_cols-1 do begin
        if (read_data(n)) then begin
          ;; read data (must be a named variable)
          tmp = data.(i)
          readu,unit,tmp
          data.(i) = tmp
          i = i + 1
        endif else begin
          ;; skip over data
          fs = fstat(unit)
          point_lun,unit,fs.cur_ptr + col_size(n)
        endelse
      endfor
      
    endif

    free_lun,unit
    return    
  end

endcase

return
end


