
;+
; NAME:
;
; sgt_compress
;
; PURPOSE:
;
; To retrieve the compression scheme used on date from a SOHO/SUMER
; observation.
;
; CATEGORY:
;
; Data handling. 
;
; CALLING SEQUENCE:
;
; c = sgt_compress(index)
; 
; INPUTS:
;
; The index structure returned from rd_sumer.
;
; OPTIONAL INPUTS:
;
; None.
;       
; KEYWORD PARAMETERS:
;
; None.
;
; OUTPUTS:
;
; The code of the compression scheme used to compress the data.
;
; OPTIONAL OUTPUTS:
;
; compar: The compression parameters used. 
;
; COMMON BLOCKS:
;
; None.
;
; SIDE EFFECTS:
;
; If the COMPRESS, COMPAR1, COMPAR2, or COMPAR3 tag names can not be
; found in the index structure passed to the routine, an error occurs.
;
; If COMPRESS equals a null string ('') then -1 is returned.
;
; RESTRICTIONS:
;
; Only the index structure may be passed as input. It may turn out to
; be useful to pass the filename as input and this option may be added
; in the future.
;
; PROCEDURE:
;
; The routine returns the values of index.gen.compress and
; index.gen.compar* from the index structure. The values of binx and
; biny have been taken directly from the primary FITS header.
;
; EXAMPLE:
;
; IDl> filename = '$SUMER_DATA/sum_960513_085055.fits'
; IDL> rd_sumer,index,data
; IDL> c = sgt_compress(index,compar=compar)
; IDL> print,c
;        5
; IDL> print,compar
;       33          89           0
;
; MODIFICATION HISTORY:
;
;   HPW 24-JUL-1996:
;   HPW 07-AUG-1996: Renamed sgt_compress.
;
;-

function sgt_compress,input,compar=compar

;; return to main level on error
on_error,1

case datatype(input,1) of

  ;; assume index structure has been passed
  'Structure': begin

    ;; get the slit name in string form
    c = sgt_tagname(input,'COMPRESS',status=status)

    if (status) then begin
      ;; parse string
      pos  = strpos(c,'<')
      if (pos ne -1) then code = fix(strmid(c,pos+1,1)) else code = -1

      ;; get compression parameters 
      ;; are there variations?
      c1 = sgt_tagname(input,'COMPAR1',status=status1)
      c2 = sgt_tagname(input,'COMPAR2',status=status2)
      c3 = sgt_tagname(input,'COMPAR3',status=status3)
      if (status1 and status2 and status3) then begin
        compar = [c1,c2,c3]
      endif else begin
        message,'Pathological index structure.'
      endelse

    endif else begin
      message,'Pathological index structure.'
    endelse
  end

  else: message,'Currently, only the index stucture may be passed as input.'
      
endcase


return,code
end
