
;+
; NAME:
;
; sgt_delta
;
; PURPOSE:
;
; To retrieve the spatial and spectral resolution of a SOHO/SUMER
; observation.
;
; CATEGORY:
;
; Data handling. 
;
; CALLING SEQUENCE:
;
; delta = sgt_delta(index)
; delta = sgt_delta(index.(i))
; 
; INPUTS:
;
; The index structure returned from rd_sumer.
;
; OPTIONAL INPUTS:
;
; None.
;       
; KEYWORD PARAMETERS:
;
; None.
;
; OUTPUTS:
;
; The spatial and spectral resolution of an observation. 
;
; OPTIONAL OUTPUTS:
;
; None.
;
; COMMON BLOCKS:
;
; None.
;
; SIDE EFFECTS:
;
; If the DELTA tag name can not be found in the index structure passed
; to the routine, an error occurs.
;
; RESTRICTIONS:
;
; Only the index structure may be passed as input. It may turn out to
; be useful to pass the filename as input and this option may be added
; in the future.
;
; PROCEDURE:
;
; The routine returns the values of index.(i).delta which have been
; taken from the binary table extension header in the FITS file.
;
; EXAMPLE:
;
; IDl> filename = '$SUMER_DATA/sum_960513_085055.fits'
; IDL> rd_sumer,index,data
; IDL> help,/str,index
; ** Structure <21fa08>, 7 tags, length=1648, refs=1:
;    W_937_803       STRUCT    -> <Anonymous> Array(1)
;    W_933_380       STRUCT    -> <Anonymous> Array(1)
;    W_930_748       STRUCT    -> <Anonymous> Array(1)
;    W_926_226       STRUCT    -> <Anonymous> Array(1)
;    W_909_000       STRUCT    -> <Anonymous> Array(1)
;    W_944_520       STRUCT    -> <Anonymous> Array(1)
;    GEN             STRUCT    -> <Anonymous> Array(1)
; IDL> delta = sgt_delta(index)
; IDL> print,delta
;    0.0444715      1.01629      1.20000
;    0.0444845      1.01659      1.20000
;    0.0444922      1.01676      1.20000
;    0.0445055      1.01707      1.20000
;    0.0445552      1.01820      1.20000
;    0.0444515      1.01583      1.20000
;
; Note that in some files, such as this one, the delta value
; corresponding to the x pointing corrdinate may be incorrect.
;
; IDL> delta = sgt_delta(index.(0))
; IDL> print,delta
;    0.0444715      1.01629      1.20000
; IDL> delta = sgt_delta(index.(6))
; % GT_DELTA: Pathological index structure.
;
; MODIFICATION HISTORY:
;
;   HPW 24-JUL-1996:
;
;   HPW 07-AUG-1996: Renamed sgt_delta.
;
;   HPW 15-OCT-1996: Modified the routine to use boost_array for array
;                    concatenation.
;
;-

function sgt_delta,input

;; return to main level on error
on_error,1

case datatype(input,1) of

  ;; assume index structure has been passed
  'Structure': begin

    ;; loop over tags and look for delta tag name in sub-structures
    num_tags = n_tags(input)
    names    = tag_names(input)
    for i=0,num_tags-1 do begin
      if (datatype(input.(i),1) eq 'Structure') then begin
        var = sgt_tagname(input.(i),'DELTA',status=status)
        if (status) then begin
          if (n_elements(delta) eq 0) then $
              delta = [var] else $
              boost_array,delta,var
        endif
      endif else begin
        if (names(i) eq 'DELTA') then begin
          var = input.(i)
          if (n_elements(delta) eq 0) then $
              delta = [var] else $
              boost_array,delta,var
        endif
      endelse
    endfor

    if n_elements(delta) eq 0 then begin
      message,'Pathological index structure.'
    endif

  end

  else: message,'Currently, only the index stucture may be passed as input.'

endcase

return,reform(delta)
end

  
