;+
; NAME:
;
; sgt_det_y
;
; PURPOSE:
;
; Returns a position array based on the index structure
;
; CATEGORY:
;
; Data handling.
;
; CALLING SEQUENCE:
;
; y = sgt_det_y(index) 
; y = sgt_det_y(index.(i))
; 
; INPUTS:
;
; Either the entire index structure, in which case an array for each
; column of data in the index is returned or part of the index
; structure.
;
; OPTIONAL INPUTS:
;
; None
;       
; KEYWORD PARAMETERS:
;
; no_flip: Don't reverse the direction of y.
;          Default no_flip=1 for .fts files, 
;                  no_flip=0 for .fits files.  
;
; OUTPUTS:
;
; The position along the slit in arcseconds.
;
; OPTIONAL OUTPUTS:
;
; None.
;
; COMMON BLOCKS:
;
; None.
;
; SIDE EFFECTS:
;
; If the necessary tag names can not be found in the index structure
; passed to the routine, an error occurs.
;
; RESTRICTIONS:
;
; None.
;
; PROCEDURE:
;
; Uses the refpix, refval, dim, and delta variables from the index
; structure to compute the position array
;
; EXAMPLE:
;
; IDL> filename = '$SUMER_DATA/sum_960513_085055.fits'
; IDL> mk_query,filename
; File: /net/solb2/soho/sumer_fits/mariska/sum_960513_085055.fits
; C      Name    Dimensions    Wmin    Wmax     Xmin     Xmax     Ymin     Ymax
; 1 W_937.803    (50,360,5)  936.69  938.91    -1.50     4.50   400.12   400.12
; 2 W_933.380    (50,360,5)  932.27  934.49    -1.50     4.50   400.12   400.12
; 3 W_930.748    (50,360,5)  929.64  931.86    -1.50     4.50   400.12   400.12
; 4 W_926.226    (50,360,5)  925.11  927.34    -1.50     4.50   400.12   400.12
; 5 W_909.000    (50,360,5)  907.89  910.11    -1.50     4.50   400.12   400.12
; 6 W_944.520    (50,360,5)  943.41  945.63    -1.50     4.50   400.12   400.12
; IDL> rd_sumer,index,data
; IDL> y = sgt_det_y(index)
; IDL> help,y
; Y            FLOAT     = Array(360, 6)
; IDL> plot,y(*,0),data.(0)(25,*,0)
;
; MODIFICATION HISTORY:
;
;   HPW 09-AUG-1996: 
;   TAK 06-JAN-1999: Default no_flip=1 for .fts files, 
;                            no_flip=0 for .fits files.  
;
;-

function sgt_det_y,input,no_flip=no_flip

;; return to main level on error
on_error,1

case datatype(input,1) of

  ;; assume index structure has been passed
  'Structure': begin

    delta   = sgt_delta(input)  
    dims    = sgt_dims(input)
    crpix   = sgt_crpix(input) 
    solar_y = sgt_solar_y(input)
    
  ;;Data has already been flipped if the file is ".FTS"
    len      = strlen(input.gen.filename)
    data_ext = strupcase(strmid(input.gen.filename,len-3,3))
    if (data_ext eq "FTS") and (n_elements(no_flip) eq 0) then no_flip=1b
    if keyword_set(no_flip) then s = 1 else s = -1

    delta_y = s*delta(1,*) 
    dim_y   = dims(1,*)
    crpix_y = crpix(1,*) 

    ;; number of exposures and number of columns
    n_cols = n_elements(solar_y(0,*)) > 1
    n_exps = n_elements(solar_y(*,0)) > 1

    ;; check uniqueness of slit lengths
    n_u_slit = n_elements(uniq(dim_y,sort(dim_y)))
    n_slit   = dim_y(0)

    if (n_u_slit eq 1) then begin
      y = fltarr(n_slit,n_exps,n_cols)
      for j=0,n_cols-1 do begin
        rel_slit = delta_y(j)*(findgen(n_slit)-crpix_y(j))
        for i=0,n_exps-1 do begin
          y(*,i,j) = solar_y(i,j) + rel_slit
        endfor
      endfor

    endif else begin
      message,'The dimensions of the data arrays are not similar.',/continue
      message,'Please pass sub-subsections of the index structure.'
    endelse
  end

  else: message,'Currently, only the index stucture may be passed as input.'

endcase

return,reform(y)
end



