
;+
; NAME:
;
; sgt_detector
;
; PURPOSE:
;
; To retrieve the detector used for a SOHO/SUMER observation.
;
; CATEGORY:
;
; Data handling. 
;
; CALLING SEQUENCE:
;
; dector = sgt_detector(index)
; 
; INPUTS:
;
; The index structure returned from rd_sumer.
;
; OPTIONAL INPUTS:
;
; None.
;       
; KEYWORD PARAMETERS:
;
; Text: Returns a string (A, B, or RSC) rather than a number (1, 2, or 3).
;
; OUTPUTS:
;
; The detector used for a SOHO/SUMER observation in either string (A,
; B, or RSC) or number (1, 2, or 3) format.
;
; OPTIONAL OUTPUTS:
;
; None.
;
; COMMON BLOCKS:
;
; None.
;
; SIDE EFFECTS:
;
; If the detector tag name can not be found in the index structure
; passed to the routine, an error occurs.
;
; RESTRICTIONS:
;
; Only the index structure may be passed as input. It may turn out to
; be useful to pass the filename as input and this option may be added
; in the future.
;
; PROCEDURE:
;
; The routine returns the values of index.gen.detector from the index
; structure. The value for detector has been taken directly from the
; primary FITS header.
;
; EXAMPLE:
;
; IDl> filename = '$SUMER_DATA/sum_960513_085055.fits'
; IDL> rd_sumer,index,data
; IDL> detector = sgt_detector(index)
; IDL> print,detector
;      1
; IDL> detector = sgt_detector(index,/text)
; IDL> print,detector
;      A
;
; MODIFICATION HISTORY:
;
;   HPW 24-JUL-1996:
;   HPW 07-AUG-1996: Renamed sgt_detector.
;
;-

function sgt_detector,input,text=text

;; return to main level on error
on_error,1

case datatype(input,1) of

  ;; assume index structure has been passed
  'Structure': begin

    d = sgt_tagname(input,'DETECTOR',status=status)

    if (status) then begin
      if not(keyword_set(text)) then begin
        pos      = strpos(d,'<')
        detector = fix(strmid(d,pos+1,1))
      endif else begin
        pos      = strpos(d,'>')
        detector = strtrim((strmid(d,pos+1,4)),2)
      endelse
    endif else begin
      message,'Pathological index structure.'
    endelse
  end

  else: message,'Currently, only the index stucture may be passed as input.'
      
endcase

return,detector
end
