
;+
; NAME:
;
; sgt_dims
;
; PURPOSE:
;
; To retrieve the spatial and spectral dimensions of a SOHO/SUMER
; observation.
;
; CATEGORY:
;
; Data handling. 
;
; CALLING SEQUENCE:
;
; dims = sgt_dims(index)
; dims = sgt_dims(index.(i))
; 
; INPUTS:
;
; The index structure returned from rd_sumer.
;
; OPTIONAL INPUTS:
;
; None.
;       
; KEYWORD PARAMETERS:
;
; None.
;
; OUTPUTS:
;
; The spatial and spectral dimensions of the array which holds data
; from an observation. If the entire index structure is passed then
; the dimensions of all of the data columns are used. If only a subset
; of the index structure is passed then only the dimensions of those
; data columds are returned.
;
; OPTIONAL OUTPUTS:
;
; None.
;
; COMMON BLOCKS:
;
; None.
;
; SIDE EFFECTS:
;
; If the DIMS tag name can not be found in the index structure passed
; to the routine, an error occurs.
;
; RESTRICTIONS:
;
; Only the index structure may be passed as input. It may turn out to
; be useful to pass the filename as input and this option may be added
; in the future.
;
; PROCEDURE:
;
; The routine returns the values of index.(i).dims which have been
; taken from the binary table extension header in the FITS file.
;
; EXAMPLE:
;
; IDl> filename = '$SUMER_DATA/sum_960513_085055.fits'
; IDL> rd_sumer,index,data
; IDL> help,/str,index
; ** Structure <21fa08>, 7 tags, length=1648, refs=1:
;    W_937_803       STRUCT    -> <Anonymous> Array(1)
;    W_933_380       STRUCT    -> <Anonymous> Array(1)
;    W_930_748       STRUCT    -> <Anonymous> Array(1)
;    W_926_226       STRUCT    -> <Anonymous> Array(1)
;    W_909_000       STRUCT    -> <Anonymous> Array(1)
;    W_944_520       STRUCT    -> <Anonymous> Array(1)
;    GEN             STRUCT    -> <Anonymous> Array(1)
; IDL> dims = sgt_dims(index)
; IDL> print,dims
;      50     360       5
;      50     360       5
;      50     360       5
;      50     360       5
;      50     360       5
;      50     360       5
; IDL> dims = sgt_dims(index.(0))
; IDL> print,dims
;      50     360       5
; IDL> dims = sgt_dims(index.(6))
; % GT_DIMS: Pathological index structure.
;
; MODIFICATION HISTORY:
;
;   HPW 24-JUL-1996:
;
;   HPW 07-AUG-1996: Renamed sgt_dims.
;
;   HPW 15-OCT-1996: Modified the routine to use boost_array for array
;                    concatenation.
;
;-

function sgt_dims,input

;; return to main level on error
on_error,1

case datatype(input,1) of

  ;; assume index structure has been passed
  'Structure': begin

    ;; loop over tags and look for dims tag name in sub-structures
    num_tags = n_tags(input)
    names    = tag_names(input)
    for i=0,num_tags-1 do begin
      if (datatype(input.(i),1) eq 'Structure') then begin
        var = sgt_tagname(input.(i),'DIMS',status=status)
        if (status) then begin
          if (n_elements(dims) eq 0) then $
              dims = [var] else $
              boost_array,dims,var
        endif
      endif else begin
        if (names(i) eq 'DIMS') then begin
          var = input.(i)
          if (n_elements(dims) eq 0) then $
              dims = [var] else $
              boost_array,dims,var
        endif
      endelse
    endfor

    if n_elements(dims) eq 0 then begin
      message,'Pathological index structure.'
    endif

  end

  else: message,'Currently, only the index stucture may be passed as input.'

endcase

return,reform(dims)
end

  
