
;+
; NAME:
;
; sgt_exposure
;
; PURPOSE:
;
; To retrieve an exposure of a SOHO/SUMER detector from a data
; structure or fits file.
;
; CATEGORY:
;
; Data handling.
;
; CALLING SEQUENCE:
;
; exposure = sgt_exposure(data,data_col=1,data_exp=1)
; exposure = sgt_exposure(filename,data_col=1,data_exp=1)
; exposure = sgt_exposure(index,data,data_col=1,data_exp=1,lambda=lambda)
; 
; INPUTS:
;
; Either the data structure returned from rd_sumer or a file name. If
; the data column and detector exposure are not specified the first
; exposure from the first data column is returned.
;
; OPTIONAL INPUTS:
;
; If the wavelength range is also desired then the index structure
; must also be passed. Note that in this case the index variable
; PRECEEDES the data variable.
;       
; KEYWORD PARAMETERS:
;
; data_col: The column from which the detector exposure is to be
;           returned. The columns are number from 1 to the number of
;           data columns.
; 
; data_exp: The detector exposure to be returned. The exposures are
;           numbered from 1 to the number of exposures within a data
;           column.
;
; OUTPUTS:
;
; The data from an exposure of the detector.
;
; OPTIONAL OUTPUTS:
;
; An array corresponding to the wavelength (dispersion) direction of
; the detector.
;
; COMMON BLOCKS:
;
; None.
;
; SIDE EFFECTS:
;
; None.
;
; RESTRICTIONS:
;
; Only a single exposure may be specified. 
;
; PROCEDURE:
;
; If the data structure is passed then the routine simply returns the
; specified data. If a file name is passed then the specified data
; column is first read with rd_sumer.
;
; EXAMPLE:
;
; Here the fifth exposure from the first data column is returned.
;
; IDL> filename = 'sum_960513_085055.fits'
; IDL> mk_query,filename
;  File: sum_960513_085055.fits
; C      Name    Dimensions    Wmin    Wmax     Xmin     Xmax     Ymin     Ymax
; 1 W_937.803    (50,360,5)  936.69  938.91    -1.50     4.50   400.12   400.12
; 2 W_933.380    (50,360,5)  932.27  934.49    -1.50     4.50   400.12   400.12
; 3 W_930.748    (50,360,5)  929.64  931.86    -1.50     4.50   400.12   400.12
; 4 W_926.226    (50,360,5)  925.11  927.34    -1.50     4.50   400.12   400.12
; 5 W_909.000    (50,360,5)  907.89  910.11    -1.50     4.50   400.12   400.12
; 6 W_944.520    (50,360,5)  943.41  945.63    -1.50     4.50   400.12   400.12
; IDL> dat = sgt_exposure(filename,data_col=1,data_exp=5)
;  or
; IDL> rd_sumer,filename,index,data
; IDL> dat = sgt_exposure(data,data_col=1,data_exp=5)
;  or
; IDL> rd_sumer,filename,index,data
; IDL> dat = sgt_exposure(index,data,data_col=1,data_exp=5,lambda=lambda)
;
; MODIFICATION HISTORY:
;
;   HPW 08-AUG-1996:
;
;   HPW 30-DEC-1996: Fixed a bug that made it impossible to read the
;                    specified exposure.
;
;-

function sgt_exposure,input1,input2,data_col=input_data_col,$
                      data_exp=input_data_exp,lambda=lambda

;; return to main level on error
on_error,1

;; ----------------------------------------------------------------------
;; Check input parameters
;; ----------------------------------------------------------------------

case n_params() of
  1: begin
    ;; either data structure or filename
    input = input1	
  end
  2: begin
    ;; must be data structure AND index structure
    input = input2
    index = input1
  end
endcase

;; ----------------------------------------------------------------------
;; Check user input and assign default values if necessary
;; ----------------------------------------------------------------------

s = strtrim(string(1),2)
if not(n_elements(input_data_col)) then begin
  message,'data_col not specified, using default data_col = '+s,/continue
  data_col = 1 
endif else begin
  if (n_elements(input_data_col) gt 1) then begin
    message,'only a single data column may be specified'
  endif else data_col = input_data_col(0)
endelse

if not(n_elements(input_data_exp)) then begin
  message,'data_exp not specified, using default data_exp = '+s,/continue
  data_exp = 1 
endif else begin
  if (n_elements(input_data_exp) gt 1) then begin
    message,'only a single dectector exposure may be specified'
  endif else data_exp = input_data_exp(0)
endelse

;; ----------------------------------------------------------------------
;; Read data column
;; ----------------------------------------------------------------------
    
case datatype(input,1) of

  ;; assume data structure has been passed
  'Structure': begin

    ;; check data_col
    num_tags = n_tags(input)
    if ((data_col lt 1) or (data_col gt num_tags)) then begin
      s = strtrim(string(data_col),2)
      message,"data_col improperly specified: data_col = "+s,/CONTINUE
      message,"must be a number between 1 and the number of data columns"
    endif
    
    ;; check data_exp
    data_size = size(input.(data_col-1))
    if (n_elements(data_size) eq 5) then num_exp=1 else num_exp=data_size(3)
    if ((data_exp lt 1) or (data_exp gt num_exp)) then begin
      s = strtrim(string(data_exp),2)
      message,"data_exp improperly specified: data_exp = "+s,/CONTINUE
      message,"must be a number between 1 and the number of detector exposures"
    endif

    exposure = input.(data_col-1)(*,*,data_exp-1)

    if (n_elements(index) gt 0) then lambda = sgt_wave(index.(data_col-1))

  end

  ;; assume filename has been passed
  'String': begin

    ;; check for file
    if (n_elements(input) gt 1) then $
        message,'Only a single file name may be specified'

    chk_file = findfile(input)
    match    = where(chk_file eq '',count)
    
    if ((count eq 0) and (n_elements(chk_file) eq 1)) then begin

      rd_sumer,chk_file(0),index,data,data_cols=[data_col]

      ;; check data_col
      num_tags = n_tags(data)
      if ((data_col lt 1) or (data_col gt num_tags)) then begin
        s = strtrim(string(data_col),2)
        message,"data_col improperly specified: data_col = "+s,/CONTINUE
        message,"must be a number between 1 and the number of data columns"
      endif
      
      ;; check data_exp
      data_size = size(data.(0))
      if (n_elements(data_size) eq 5) then num_exp=1 else num_exp=data_size(3)
      if ((data_exp lt 1) or (data_exp gt num_exp)) then begin
        s = strtrim(string(data_exp),2)
        message,"data_exp improperly specified: data_exp = "+s,/CONTINUE
        message,"must be a number between 1 and the number of detector"+$
            "exposures"
      endif

      lambda   = sgt_wave(index.(0))
      exposure = data.(0)(*,*,data_exp-1)

    endif else begin
      if (count gt 0) then $
          message,"Specified file not found: "+input $
      else if (n_elements(chk_file) gt 1) then $
          message,"Only a single file name may be specified, no wildcards"
    endelse
      
  end
    
  else: begin
    message,'Only a data structure or a filename may be passed as input'
  end
    
endcase


return,exposure
end

  
