
;+
; NAME:
;
; sgt_refpix
;
; PURPOSE:
;
; To retrieve the reference pixels used to co-register the data with
; the detector face.
;
; CATEGORY:
;
; Data handling. 
;
; CALLING SEQUENCE:
;
; refpix = sgt_refpix(index)
; refpix = sgt_refpix(index.(i))
; 
; INPUTS:
;
; The index structure returned from rd_sumer.
;
; OPTIONAL INPUTS:
;
; None.
;       
; KEYWORD PARAMETERS:
;
; None.
;
; OUTPUTS:
;
; The wavelength and spatial reference pixels. If the entire index
; structure is passed then all of reference pixels are returned. If
; the index structure corresponding to a single column of data is
; passed then only those reference pixels are returned.
;
; Note that the reference pixel returned corresponds to the
; co-pixel+1.  
;
; OPTIONAL OUTPUTS:
;
; None.
;
; COMMON BLOCKS:
;
; None.
;
; SIDE EFFECTS:
;
; If the REFPIX tag name can not be found in the index structure
; passed to the routine, an error occurs.
;
; RESTRICTIONS:
;
; Only the index structure may be passed as input. It may turn out to
; be useful to pass the filename as input and this option may be added
; in the future.
;
; PROCEDURE:
;
; The routine returns the values of index.(i).refpix which have been
; taken from the binary table extension header in the FITS file.
;
; EXAMPLE:
;
; IDl> filename = '$SUMER_DATA/sum_960513_085055.fits'
; IDL> rd_sumer,index,data
; IDL> help,/str,index
; ** Structure <21fa08>, 7 tags, length=1648, refs=1:
;    W_937_803       STRUCT    -> <Anonymous> Array(1)
;    W_933_380       STRUCT    -> <Anonymous> Array(1)
;    W_930_748       STRUCT    -> <Anonymous> Array(1)
;    W_926_226       STRUCT    -> <Anonymous> Array(1)
;    W_909_000       STRUCT    -> <Anonymous> Array(1)
;    W_944_520       STRUCT    -> <Anonymous> Array(1)
;    GEN             STRUCT    -> <Anonymous> Array(1)
; IDL> refpix = sgt_refpix(index)
; IDL> print,refpix
;      715.000      179.000      1.00000
;      616.000      179.000      1.00000
;      557.000      179.000      1.00000
;      456.000      179.000      1.00000
;      72.0000      179.000      1.00000
;      865.000      179.000      1.00000
; IDL> refpix = sgt_refpix(index.(0))
; IDL> print,refpix
;      715.000      179.000      1.00000
; IDL> refpix = sgt_refpix(index.(6))
; % GT_REFPIX: Pathological index structure.
;
; MODIFICATION HISTORY:
;
;   HPW 24-JUL-1996:
;
;   HPW 07-AUG-1996: Renamed sgt_refpix.
;
;   HPW 15-OCT-1996: Modified the routine to use boost_array for array
;                    concatenation.
;
;-

function sgt_refpix,input

;; return to main level on error
on_error,1

case datatype(input,1) of

  ;; assume index structure has been passed
  'Structure': begin

    ;; loop over tags and look for refpix tag name in sub-structures
    num_tags = n_tags(input)
    names    = tag_names(input)
    for i=0,num_tags-1 do begin
      if (datatype(input.(i),1) eq 'Structure') then begin
        var = sgt_tagname(input.(i),'REFPIX',status=status)
        if (status) then begin
          if (n_elements(refpix) eq 0) then $
              refpix = [var] else $
              boost_array,refpix,var
        endif
      endif else begin
        if (names(i) eq 'REFPIX') then begin
          var = input.(i)
          if (n_elements(refpix) eq 0) then $
              refpix = [var] else $
              boost_array,refpix,var
        endif
      endelse
    endfor

    if n_elements(refpix) eq 0 then begin
      message,'Pathological index structure.'
    endif

  end

  else: message,'Currently, only the index stucture may be passed as input.'

endcase

return,reform(refpix)
end

  
