
;+
; NAME:
;
; sgt_slit
;
; PURPOSE:
;
; To retrieve the slit used for an SOHO/SUMER observation.
;
; CATEGORY:
;
; Data handling. 
;
; CALLING SEQUENCE:
;
; slit = sgt_slit(index)
; 
; INPUTS:
;
; The index structure returned from rd_sumer.
;
; OPTIONAL INPUTS:
;
; None.
;       
; KEYWORD PARAMETERS:
;
; Text: Returns a string describing the slit instead of a numerical
;       code.
;
; OUTPUTS:
;
; An integer code or string which identifies the slit used in an
; observation.
;
; OPTIONAL OUTPUTS:
;
; None.
;
; COMMON BLOCKS:
;
; None.
;
; SIDE EFFECTS:
;
; If the SLIT tag name can not be found in the index structure passed
; to the routine, an error occurs.
;
; RESTRICTIONS:
;
; Only the index structure may be passed as input. It may turn out to
; be useful to pass the filename as input and this option may be added
; in the future.
;
; PROCEDURE:
;
; The routine parses the value of index.gen.slit from the index
; structure. The value for slit has been taken directly from the
; primary FITS header.
;
; EXAMPLE:
;
; IDl> filename = '$SUMER_DATA/sum_960513_085055.fits'
; IDL> rd_sumer,index,/nodata
; IDL> slit = sgt_slit(index)
; IDL> print,slit
;      2
; IDL> slit = sgt_slit(index,/text)
; IDL> print,slit
; <2> 1.0 * 300 centered
;
; MODIFICATION HISTORY:
;
;   HPW 24-JUL-1996:
;   HPW 07-AUG-1996: Renamed sgt_slit.
;
;-

function sgt_slit,input,text=text

;; return to main level on error
on_error,1

case datatype(input,1) of

  ;; assume index structure has been passed
  'Structure': begin

    ;; get the slit name in string form
    s = sgt_tagname(input,'SLIT',status=status)
    if (status) then begin
      if not(keyword_set(text)) then begin
        pos  = strpos(s,'<')
        slit = fix(strmid(s,pos+1,1))
      endif else slit = strtrim(s,2)
    endif else begin
      message,'Pathological index structure.'
    endelse
  end

  else: message,'Currently, only the index stucture may be passed as input.'
      
endcase

return,slit
end
