
;+
; NAME:
;
; sgt_solar_x
;
; PURPOSE:
;
; To retrieve the x corrdinate of the pointing of a SOHO/SUMER
; observation.
;
; CATEGORY:
;
; Data handling. 
;
; CALLING SEQUENCE:
;
; solar_x = sgt_solar_x(index)
; solar_x = sgt_solar_x(index.(i))
; 
; INPUTS:
;
; The index structure returned from rd_sumer.
;
; OPTIONAL INPUTS:
;
; None.
;       
; KEYWORD PARAMETERS:
;
; None.
;
; OUTPUTS:
;
; The x coordinate of the pointing in arcsec. If the entire index
; structure is passed then all of x coordinates are returned. If the
; index structure corresponding to a single column of data is passed
; then only those x coordinates are returned.
;
; OPTIONAL OUTPUTS:
;
; None.
;
; COMMON BLOCKS:
;
; None.
;
; SIDE EFFECTS:
;
; If the SOLAR_X tag name can not be found in the index structure
; passed to the routine, an error occurs.
;
; RESTRICTIONS:
;
; Only the index structure may be passed as input. It may turn out to
; be useful to pass the filename as input and this option may be added
; in the future.
;
; PROCEDURE:
;
; The routine returns the values of index.(i).solar_x which have been
; taken from the binary table extension header in the FITS file.
;
; EXAMPLE:
;
; IDl> filename = '$SUMER_DATA/sum_960513_085055.fits'
; IDL> rd_sumer,index,data
; IDL> help,/str,index
; ** Structure <21fa08>, 7 tags, length=1648, refs=1:
;    W_937_803       STRUCT    -> <Anonymous> Array(1)
;    W_933_380       STRUCT    -> <Anonymous> Array(1)
;    W_930_748       STRUCT    -> <Anonymous> Array(1)
;    W_926_226       STRUCT    -> <Anonymous> Array(1)
;    W_909_000       STRUCT    -> <Anonymous> Array(1)
;    W_944_520       STRUCT    -> <Anonymous> Array(1)
;    GEN             STRUCT    -> <Anonymous> Array(1)
; IDL> solar_x = sgt_solar_x(index)
; IDL> print,solar_x
;     -1.50000      0.00000      1.50000      3.00000      4.50000
;     -1.50000      0.00000      1.50000      3.00000      4.50000
;     -1.50000      0.00000      1.50000      3.00000      4.50000
;     -1.50000      0.00000      1.50000      3.00000      4.50000
;     -1.50000      0.00000      1.50000      3.00000      4.50000
;     -1.50000      0.00000      1.50000      3.00000      4.50000
; IDL> solar_x = sgt_solar_x(index.(0))
; IDL> print,solar_x
;     -1.50000      0.00000      1.50000      3.00000      4.50000
; IDL> solar_x = sgt_solar_x(index.(6))
; % GT_SOLAR_X: Pathological index structure.
;
; MODIFICATION HISTORY:
;
;   HPW 24-JUL-1996:
;
;   HPW 07-AUG-1996: Renamed sgt_solar_x.
;
;   HPW 15-OCT-1996: Modified the routine to use boost_array for array
;                    concatenation.
;
;-

function sgt_solar_x,input

;; return to main level on error
on_error,1

case datatype(input,1) of

  ;; assume index structure has been passed
  'Structure': begin

    ;; loop over tags and look for solar_x tag name in sub-structures
    num_tags = n_tags(input)
    names    = tag_names(input)
    for i=0,num_tags-1 do begin
      if (datatype(input.(i),1) eq 'Structure') then begin
        var = sgt_tagname(input.(i),'SOLAR_X',status=status)
        if (status) then begin
          if (n_elements(solar_x) eq 0) then $
              solar_x = [var] else $
              boost_array,solar_x,var
        endif
      endif else begin
        if (names(i) eq 'SOLAR_X') then begin
          var = input.(i)
          if (n_elements(solar_x) eq 0) then $
              solar_x = [var] else $
              boost_array,solar_x,var
        endif
      endelse
    endfor

    if n_elements(solar_x) eq 0 then begin
      message,'Pathological index structure.'
    endif

  end

  else: message,'Currently, only the index stucture may be passed as input.'

endcase

return,reform(solar_x)
end

  
