
;+
; NAME:
;
; sgt_spectrum
;
; PURPOSE:
;
; To retrieve an SOHO/SUMER spectrum from a data structure or fits
; file.
;
; CATEGORY:
;
; Data handling.
;
; CALLING SEQUENCE:
;
; spectrum = sgt_spectrum(data,data_col=1,data_exp=1,data_spec=180)
; spectrum = sgt_spectrum(filename,data_col=1,data_exp=1,data_spec=180)
; spectrum = sgt_spectrum(index,data,data_col=1,data_exp=1,data_spec=180,$
;                         lambda=lambda)
; 
; INPUTS:
;
; Either the data structure returned from rd_sumer or a file name. If
; the data column, detector exposure, and/or spectrum are not
; specified the first spectrum from the first exposure in the first
; data column is returned.
;
; OPTIONAL INPUTS:
;
; If the wavelength range is also desired then the index structure
; must also be passed. Note that in this case the index variable
; PRECEEDES the data variable.
;
; KEYWORD PARAMETERS:
;
; data_col: The column from which the spectrum is to be returned. The
;           columns are number from 1 to the number of data columns.
; 
; data_exp: The detector exposure from which the spectrum is to be
;           returned. The exposures are numbered from 1 to the number
;           of exposures within a data column.
;
; data_spec: The spectrum to be returned. The spectra are number from
;            1 to the number of spectra (essentially the length of the
;            slit) within an exposure.
;
; OUTPUTS:
;
; A spectrum.
;
; OPTIONAL OUTPUTS:
;
; An array corresponding to the wavelength (dispersion) direction of
; the detector.
;
; COMMON BLOCKS:
;
; None.
;
; SIDE EFFECTS:
;
; None.
;
; RESTRICTIONS:
;
; Only a single spectrum may be specified.
;
; PROCEDURE:
;
; Calls sgt_exposure and then returns the specified spectrum.
;
; EXAMPLE:
;
; Here a spectrum at the center of the slit from the fifth exposure in
; the first data column is returned.
;
; IDL> filename = 'sum_960513_085055.fits'
; IDL> mk_query,filename
;  File: sum_960513_085055.fits
; C      Name    Dimensions    Wmin    Wmax     Xmin     Xmax     Ymin     Ymax
; 1 W_937.803    (50,360,5)  936.69  938.91    -1.50     4.50   400.12   400.12
; 2 W_933.380    (50,360,5)  932.27  934.49    -1.50     4.50   400.12   400.12
; 3 W_930.748    (50,360,5)  929.64  931.86    -1.50     4.50   400.12   400.12
; 4 W_926.226    (50,360,5)  925.11  927.34    -1.50     4.50   400.12   400.12
; 5 W_909.000    (50,360,5)  907.89  910.11    -1.50     4.50   400.12   400.12
; 6 W_944.520    (50,360,5)  943.41  945.63    -1.50     4.50   400.12   400.12
; IDL> spec = sgt_spectrum(filename,data_col=1,data_exp=5,data_spec=180)
;  or
; IDL> rd_sumer,filename,index,data
; IDL> spec = sgt_spectrum(data,data_col=1,data_exp=5,data_spec=180)
;  or
; IDL> rd_sumer,filename,index,data
; IDL> spec = sgt_spectrum(index,data,data_col=1,data_exp=5,data_spec=180,$
;                          lambda=lambda) 
;
; MODIFICATION HISTORY:
;
;   HPW 08-AUG-1996:
;
;-

function sgt_spectrum,input1,input2,data_col=input_data_col,$
                      data_exp=input_data_exp,$
                      data_spec=input_data_spec,lambda=lambda

;; return to main level on error
on_error,1

;; ----------------------------------------------------------------------
;; Check input parameters
;; ----------------------------------------------------------------------

case n_params() of
  1: begin
    ;; either data structure or filename
    input = input1	
  end
  2: begin
    ;; must be data structure AND index structure
    input = input2
    index = input1
  end
endcase

;; ----------------------------------------------------------------------
;; Check user input and assign default values if necessary
;; ----------------------------------------------------------------------

s = strtrim(string(1),2)
if not(n_elements(input_data_col)) then begin
  message,'data_col not specified, using default data_col = '+s,/continue
  data_col = 1 
endif else begin
  if (n_elements(input_data_col) gt 1) then begin
    message,'only a single data column may be specified'
  endif else data_col = input_data_col(0)
endelse

if not(n_elements(input_data_exp)) then begin
  message,'data_exp not specified, using default data_exp = '+s,/continue
  data_exp = 1 
endif else begin
  if (n_elements(input_data_exp) gt 1) then begin
    message,'only a single dectector exposure may be specified'
  endif else data_exp = input_data_exp(0)
endelse

if not(n_elements(input_data_spec)) then begin
  message,'data_spec not specified, using default data_spec = '+s,/continue
  data_spec = 1 
endif else begin
  if (n_elements(input_data_spec) gt 1) then begin
    message,'only a single spectrum may be specified'
  endif else data_spec = input_data_spec(0)
endelse

;; ----------------------------------------------------------------------
;; Read exposure from data file
;; ----------------------------------------------------------------------

case datatype(input,1) of

  'Structure': begin
    if (n_params() eq 2) then begin
      exposure = sgt_exposure(index,input,data_col=data_col,$
                              data_exp=data_exp,lambda=lambda)
    endif else begin
      exposure = sgt_exposure(input,data_col=data_col,data_exp=data_exp)
    endelse
  end

  'String': begin
    exposure = sgt_exposure(input,data_col=data_col,data_exp=data_exp,$
                           lambda=lambda)
  end

  else: begin
    message,'Only a data structure or a filename may be passed as input'
  end

endcase

;; ----------------------------------------------------------------------
;; Select spectrum from exposure
;; ----------------------------------------------------------------------

if (n_elements(exposure) gt 0) then begin

  ;; check data_spec
  num_slit = n_elements(exposure(0,*))
  if ((data_spec lt 1) or (data_spec gt num_slit)) then begin
    s = strtrim(string(data_spec),2)
    message,'data_spec improperly specified: data_spec = "+s,/CONTINUE
    message,"must be a number between 1 and the length of the slit"
  endif
  
  spectrum = exposure(*,data_spec-1)
  return,spectrum

endif

end
