
;+
; NAME:
;
; sgt_time
;
; PURPOSE:
;
; To retrieve the time at which a SOHO/SUMER observation began.
;
; CATEGORY:
;
; Data handling. 
;
; CALLING SEQUENCE:
;
; time = sgt_time(index)
; 
; INPUTS:
;
; The index structure returned from rd_sumer.
;
; OPTIONAL INPUTS:
;
; None.
;       
; KEYWORD PARAMETERS:
;
; DET_TIME: The start time for each exposure in the data structure.
;
; OUTPUTS:
;
; A seven element array containing the time. The elements of the array
; are the hour, minute, second, millisecond, day, month, year that an
; observation began. If the keyword DET_TIME is used the format will
; be an array of the form (7,n_exposures,n_columns).
;
; OPTIONAL OUTPUTS:
;
; None.
;
; COMMON BLOCKS:
;
; None.
;
; SIDE EFFECTS:
;
; If the DATE_OBS tag name can not be found in the index structure
; passed to the routine, an error occurs.
;
; RESTRICTIONS:
;
; Only the index structure may be passed as input. It may turn out to
; be useful to pass the filename as input and this option may be added
; in the future.
;
; PROCEDURE:
;
; The routine parses the value of index.gen.date_obs and returns it as
; a seven element array. Optially, the det_time tag in the index
; struture is used to determine the start time for each exposure.
;
; EXAMPLE:
;
; IDl> filename = '$SUMER_DATA/sum_960513_085055.fits'
; IDL> rd_sumer,index,/nodata
; IDL> time = sgt_time(index)
; IDL> print,time
;      8      50      55      30      13       5      96
;
; MODIFICATION HISTORY:
;
;   HPW 24-JUL-1996:
;   HPW 07-AUG-1996: Renamed sgt_time.
;   HPW 20-OCT-1997: Modified to return start times for each exposure.
;-

function sgt_time,input,det_time=det_time

;; return to main level on error
on_error,1

case datatype(input,1) of

  ;; assume index structure has been passed
  'Structure': begin

    ;; get starting time from index in string form
    t = sgt_tagname(input,'DATE_OBS',status=status)
    
    if (status) then begin
      ;; parse string
      yy = strmid(t,2,2)  & mm = strmid(t,5,2)  & dd = strmid(t,8,2)
      h  = strmid(t,11,2) & m  = strmid(t,14,2) & s  = strmid(t,17,2)
      ms = strmid(t,20,3)
      
      ;; [hour,minute,second,millisecond,day,month,year]
      start_time = fix([h,m,s,ms,dd,mm,yy])

      ;; find start times for each exposure
      if keyword_set(DET_TIME) then begin
        n_tags = n_tags(input)
        for n=0,n_tags-1 do begin
          det_time = sgt_tagname(input.(n),"DET_TIME",status=status)
          if status then begin
            col_time = intarr(n_elements(det_time),7)
            for m=0,n_elements(det_time)-1 do begin
              col_time(m,*) = addtime(start_time,delta_min=det_time(m)/60.0)
            endfor
            boost_array,time,transpose(col_time)
          endif
        endfor
        time = reform(time)
      endif else time = start_time

    endif else begin
      message,'Pathological index structure.'
    endelse
  end

  else: message,'Currently, only the index stucture may be passed as input.'
      
endcase

return,time
end
  
