

;+
; NAME:
;
; sgt_wave
;
; PURPOSE:
;
; Returns a wavelength array based on the index structure
;
; CATEGORY:
;
; Data handling.
;
; CALLING SEQUENCE:
;
; wave = sgt_wave(index) 
; wave = sgt_wave(index.(i))
; 
; INPUTS:
;
; Either the entire index structure, in which case the wavelength
; array is based on the first data column, or the subset of the index
; structure pertaining to a specific data column. See the routine
; rd_sumer.pro for the definition of the index structure.
;
; OPTIONAL INPUTS:
;
; None
;       
; KEYWORD PARAMETERS:
;
; None
;
; OUTPUTS:
;
; The wavelength array in Angstroms.
;
; OPTIONAL OUTPUTS:
;
; None.
;
; COMMON BLOCKS:
;
; None.
;
; SIDE EFFECTS:
;
; If the necessary tag names can not be found in the index structure
; passed to the routine, an error occurs.
;
; RESTRICTIONS:
;
; None.
;
; PROCEDURE:
;
; Uses the refpix, refval, dim, and delta variables from the index
; structure to compute the wavelgnth array
;
; EXAMPLE:
;
; IDL> filename = '$SUMER_DATA/sum_960513_085055.fits'
; IDL> mk_query,filename
; File: /net/solb2/soho/sumer_fits/mariska/sum_960513_085055.fits
; C      Name    Dimensions    Wmin    Wmax     Xmin     Xmax     Ymin     Ymax
; 1 W_937.803    (50,360,5)  936.69  938.91    -1.50     4.50   400.12   400.12
; 2 W_933.380    (50,360,5)  932.27  934.49    -1.50     4.50   400.12   400.12
; 3 W_930.748    (50,360,5)  929.64  931.86    -1.50     4.50   400.12   400.12
; 4 W_926.226    (50,360,5)  925.11  927.34    -1.50     4.50   400.12   400.12
; 5 W_909.000    (50,360,5)  907.89  910.11    -1.50     4.50   400.12   400.12
; 6 W_944.520    (50,360,5)  943.41  945.63    -1.50     4.50   400.12   400.12
; IDL> rd_sumer,index,data
; IDL> wave = sgt_wave(index)
; IDL> help,wave
; WAVE            FLOAT     = Array(50, 6)
; IDL> plot,wave(*,0),total(data.(0)(*,*,0),2)
;
; MODIFICATION HISTORY:
;
;   HPW 17-JUN-1996:
;
;   HPW 10-JUL-1996: Modified to work with new index structure.
;
;   HPW 24-JUL-1996: Modified to produce a wavelength array for each
;                    column of the index structure that is passed to
;                    the routine.
;
;   HPW 07-AUG-1996: Renamed sgt_wave.
;
;   HPW 27-FEB-1997: Fixed bug related to refpix.
;
;   HPW 21-JUL-1997: Added keywords first_order and second_order to
;                    return selected wavelength range. The default is
;                    still to return the wavelength range indicated by
;                    delta and refval.
;
;   HPW 24-MAY-1999: Fixed a bug related to generating second order
;                    wavelength scales.
;
;
;-

function sgt_wave,input,first_order=first_order,second_order=second_order

;; return to main level on error
on_error,1

case datatype(input,1) of

  ;; assume index structure has been passed
  'Structure': begin

    delta  = sgt_delta(input)  & dims   = sgt_dims(input)
    refpix = sgt_refpix(input) & refval = sgt_refval(input)

    delta_x  = delta(0,*)  & dim_x    = dims(0,*)    
    refpix_x = refpix(0,*) & refval_x = refval(0,*)

    n_dim_x = n_elements(dim_x)
    n_u_dim = n_elements(uniq(dim_x,sort(dim_x)))
    if (n_u_dim eq 1) then begin

      wave = fltarr(dim_x(0),n_dim_x)
      for i=0,n_dim_x-1 do begin

        if delta_x(i) lt 0.04 then wv2firstorder=2 else wv2firstorder=1
        if delta_x(i) lt 0.04 then wv2secondorder=1 else wv2secondorder=0.5

        if (dim_x(i) eq 1024) then v_x = refpix_x(i) else v_x = dim_x(i)/2
        wave(*,i) = (refval_x(i)-delta_x(i)*v_x)+findgen(dim_x(i))*delta_x(i)

        if keyword_set(FIRST_ORDER) then wave(*,i) = wave(*,i)*wv2firstorder
        if keyword_set(SECOND_ORDER) then wave(*,i) = wave(*,i)*wv2secondorder

      endfor

    endif else begin
      message,'The dimensions of the data arrays are notsimilar.',/continue
      message,'Please pass sub-subsections of the index structure.'
    endelse
  end

  else: message,'Currently, only the index stucture may be passed as input.'

endcase

return,reform(wave)
end
