
;+
; NAME:
;
; wrt_sumer
;
; PURPOSE:
;
; To write the data and index structures returned from rd_sumer to
; genx files. These files may also be read with rd_sumer.
;
; CATEGORY:
;
; Data handling.
;
; CALLING SEQUENCE:
;
; wrt_sumer,index,data
; wrt_sumer,index,data,file_out=filename,dir_out=directory
; 
; INPUTS:
;
; The index and data structure returned from rd_sumer.
;
; OPTIONAL INPUTS:
;
; file_out: The name of the file to be written. The default is to use
;           the original SUMER FITS file name with the extension
;           '.genx' in place of '.fits'.
;
; dir_out : The directory where the file is to be written. The default
;           is to write to the current directory
;
; KEYWORD PARAMETERS:
;
; verbose: Prints the output file name to the screen.
;
; OUTPUTS:
;
; A generic file using the xdr format.
;
; OPTIONAL OUTPUTS:
;
; None.
;
; COMMON BLOCKS:
;
; None.
;
; SIDE EFFECTS:
;
; Any existing files with the same name are overwriten.
;
; RESTRICTIONS:
;
; Note that this file format may be replaced with something better!
;
; PROCEDURE:
;
; Uses unformatted writes to save the data to a file. Uses wrt_str to
; save the index structure.
;
; EXAMPLE:
;
; The following would save the data and index structures in the file
; sum_960516_052729.genx in the current directory:
; IDL> filename = "sum_960516_052729.fits" 
; IDL> rd_sumer,filename,index,data 
; IDL> wrt_sumer,index,data
;
; MODIFICATION HISTORY:
;
;   HPW 12-AUG-1996:
;
;   HPW 07-OCT-1996: Modified the routine to write a new format of
;                    .genx files which allows the nodata switch on
;                    rd_sumer to work properly.
;
;   HPW 07-OCT-1996: Fixed a bug related to the naming of files.
;
;   HPW 17-JUL-1997: Modified file format to facilitate reading
;                    individual data columns.
;
;-

pro wrt_sumer,index,data,file_out=file_out,dir_out=dir_out,$
              verbose=verbose

;; ----------------------------------------------------------------------
;; Check user input
;; ----------------------------------------------------------------------

if (datatype(index,1) ne 'Structure') then $
    message,'Index is not a structure: data type = '+datatype(index,1)
if (datatype(data,1)  ne 'Structure') then $
    message,'Data is not a structure: data type = '+datatype(data,1)

;; ----------------------------------------------------------------------
;; Determine output file name
;; ----------------------------------------------------------------------

;; create file name if it isn't specified
if not(keyword_set(file_out)) then begin
  output_file = strmid(strlowcase(index.gen.filename),0,17)+'.genx'
endif else begin
  output_file = file_out
endelse

;; set output directory
if keyword_set(dir_out) then begin
  output_file = concat_dir(dir_out,output_file)
endif

;; print output file name
if keyword_set(verbose) then print,' Writing to '+ output_file

;; ----------------------------------------------------------------------
;; Output to genx file
;; ----------------------------------------------------------------------

;; open file for writing
openw,unit,output_file,/xdr,/get_lun

;; version number - dangerous! 0,1,2 are used by Yohkoh
version = 998l
writeu,unit,version

;; ----------------------------------------------------------------------
;; INDEX
;; ----------------------------------------------------------------------

;; write definition of index structure
wrt_str,index,unit

;; write the index 
writeu,unit,index

;; ----------------------------------------------------------------------
;; DATA
;; ----------------------------------------------------------------------

;; write definition of data structure

names = tag_names(data)
n_col = n_elements(names) 
for n=0,n_col-1 do begin
  arr_size = size(data.(n))
  n_x      = arr_size(1)
  n_y      = arr_size(2)
  if arr_size(0) eq 3 then n_exp = arr_size(3) else n_exp = 1

  tag_name = (tag_names(data))(n)
  n_char   = strlen(tag_name)

  if n eq 0 then writeu,unit,long(n_col)  ;; total number of data columns

  writeu,unit,long(n_x)		;; number of wavelength pixels
  writeu,unit,long(n_y)		;; number of spatial pixels
  writeu,unit,long(n_exp)	;; number of exposures
  writeu,unit,long(n_char)	;; length of tag name
  writeu,unit,tag_name		;; tag name
endfor

;; write the data
for n=0,n_col-1 do begin
  writeu,unit,data.(n)
endfor

;; close file
free_lun,unit

return
end
